#############################################################################
# One class SVM using the German credit data
#
# Dependency: require(RevoScaleR)
# Note: internet access required for initial download of data from
# http://archive.ics.uci.edu/ml/machine-learning-databases/statlog/german/
#############################################################################

library("MicrosoftML")

source(system.file("extdata/german_credit.R", package = "MicrosoftML"))
germanCredit <- getGermanCreditData(download = TRUE, sampleDataDir = NULL, createDir = TRUE)
gcTrain <- germanCredit$germanCreditTrain
gcTest <- germanCredit$germanCreditTest
rm(germanCredit)

# Remove observations with bad credit from training data set
gcTrain <- gcTrain[gcTrain$Bad == FALSE,]
gcTrain$Bad <- NULL

# rxOneClassSvm anomaly detection
model <- rxOneClassSvm(formula = ~., data = gcTrain)

# Test the trained model
scoreDF <- rxPredict(model, data = gcTest,
    extraVarsToWrite = c("Bad"))
minScore <- min(scoreDF$Score)
maxScore <- max(scoreDF$Score)
scoreDF$ScoreNormalized <- (scoreDF$Score - minScore) / (maxScore - minScore)

# Evaluate the performance
roc <- rxRoc(actualVarName = "Bad", predVarNames = "ScoreNormalized", data = scoreDF)
rxAuc(roc)
plot(roc)