#############################################################################
# Demonstrate the usage of rxFastTrees on the adult dataset from UCI repository.
#
# Dependency: require(RevoScaleR)
# Note: internet access required for initial download of data from
# http://archive.ics.uci.edu/ml/machine-learning-databases/adult/
#############################################################################

library("MicrosoftML")

# Source the file containing the 'getAdultData' function.
source(system.file("extdata/adult.R", package = "MicrosoftML"))

adult <- getAdultData(download = TRUE, sampleDataDir = NULL, createDir = TRUE)
adult_test <- adult$adult_test
adult_train <- adult$adult_train
rm(adult)

# Read the information about the variables
rxGetVarInfo(adult_train)

# Train with rxFastTrees (gradient tree boosting)
model <- rxFastTrees(IsOver50K ~ age + sex + education + hours.per.week,
    data = adult_train)
summary(model)

# Test the trained model
scoreDF <- rxPredict(model, data = adult_test,
    extraVarsToWrite = c("IsOver50K", "education"))

# Plot the ROC curve
roc <- rxRoc(actualVarName = "IsOver50K", predVarNames = "Probability",
    data = scoreDF)
rxAuc(roc)