# Create an ensemble of regression rxFastTrees models

# use xdf data source
dataFile <- file.path(rxGetOption("sampleDataDir"), "claims4blocks.xdf")
rxGetInfo(dataFile, getVarInfo = TRUE, getBlockSizes = TRUE)
form <- cost ~ age + type + number

rxSetComputeContext("localpar")
rxGetComputeContext()

# build an ensemble model that contains three 'rxFastTrees' models with different parameters
ensemble <- rxEnsemble(
    formula = form,
    data = dataFile,
    type = "regression",
    trainers = list(fastTrees(), fastTrees(numTrees = 60), fastTrees(learningRate = 0.1)), #a list of trainers with their arguments.
    replace = TRUE # Indicates using a bootstrap sample for each trainer
    )
    
# use text data source
colInfo <- list(DayOfWeek = list(type = "factor", levels = c("Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday")))

source <- system.file("SampleData/AirlineDemoSmall.csv", package = "RevoScaleR")
data <- RxTextData(source, missingValueString = "M", colInfo = colInfo)

# When 'distributed' is TRUE distributed data source is created
distributed <- FALSE
if (distributed) {
    bigDataDirRoot <- "/share"
    inputDir <- file.path(bigDataDirRoot, "AirlineDemoSmall")
    rxHadoopMakeDir(inputDir)
    rxHadoopCopyFromLocal(source, inputDir)
    hdfsFS <- RxHdfsFileSystem()
    data <- RxTextData(file = inputDir, missingValueString = "M", colInfo = colInfo, fileSystem = hdfsFS)
}

# When 'distributed' is TRUE training is distributed
if (distributed) {
    cc <- rxSetComputeContext(RxSpark())
} else {
    cc <- rxGetComputeContext()
}

ensemble <- rxEnsemble(
    formula = ArrDelay ~ DayOfWeek,
    data = data,
    type = "regression",
    trainers = list(fastTrees(), fastTrees(numTrees = 60), fastTrees(learningRate = 0.1)), # The ensemble will contain three 'rxFastTrees' models
    replace = TRUE # Indicates using a bootstrap sample for each trainer
    )

# Change the compute context back to previous for scoring
rxSetComputeContext(cc)

# Put score and model variables in data frame
scores <- rxPredict(ensemble, data = data, writeModelVars = TRUE)

# Plot actual versus predicted values with smoothed line
rxLinePlot(Score ~ ArrDelay, type = c("p", "smooth"), data = scores)
