# Train a binary classiication model with rxFastLinear
res1 <- rxFastLinear(isCase ~ age + parity + education + spontaneous + induced,
                  transforms = list(isCase = case == 1),
                  data = infert,
                  type = "binary")
# Print a summary of the model
summary(res1)

# Score to a data frame
scoreDF <- rxPredict(res1, data = infert, 
    extraVarsToWrite = "isCase")
    
# Compute and plot the Radio Operator Curve and AUC
roc1 <- rxRoc(actualVarName = "isCase", predVarNames = "Probability", data = scoreDF) 
plot(roc1)
rxAuc(roc1)

#########################################################################
# rxFastLinear Regression

# Create an xdf file with the attitude data
myXdf <- tempfile(pattern = "tempAttitude", fileext = ".xdf")
rxDataStep(attitude, myXdf, rowsPerRead = 50, overwrite = TRUE)
myXdfDS <- RxXdfData(file = myXdf)
    
attitudeForm <- rating ~ complaints + privileges + learning + 
    raises + critical + advance

# Estimate a regression model with rxFastLinear 
res2 <- rxFastLinear(formula = attitudeForm,  data = myXdfDS, 
    type = "regression")

# Score to data frame
scoreOut2 <- rxPredict(res2, data = myXdfDS, 
    extraVarsToWrite = "rating")

# Plot the rating versus the score with a regression line
rxLinePlot(rating~Score, type = c("p","r"), data = scoreOut2)

# Clean up   
file.remove(myXdf)
