# Simulate some simple data
set.seed(7)
numRows <- 200
normalData <- data.frame(day = 1:numRows)
normalData$pageViews = runif(numRows, min = 10, max = 1000) + .5 * normalData$day
testData <- data.frame(day = 1:numRows)
# The test data has outliers above 1000
testData$pageViews = runif(numRows, min = 10, max = 1400) + .5 * testData$day

train <- function(kernelFunction, args=NULL) {
    model <- rxOneClassSvm(formula = ~pageViews + day, data = normalData,
    kernel = kernelFunction(args))
    scores <- rxPredict(model, data = testData, writeModelVars = TRUE)
    scores$groups = scores$Score > 0
    scores
}
display <- function(scores) {
    print(sum(scores$groups))
    rxLinePlot(pageViews ~ day, data = scores, groups = groups, type = "p",
     symbolColors = c("red", "blue"))
}
scores <- list()
scores$rbfKernel <- train(rbfKernel)
scores$linearKernel <- train(linearKernel)
scores$polynomialKernel <- train(polynomialKernel, (a = .2))
scores$sigmoidKernel <- train(sigmoidKernel)
display(scores$rbfKernel)
display(scores$linearKernel)
display(scores$polynomialKernel)
display(scores$sigmoidKernel)
