# Copyright (c) Microsoft Corporation.  All rights reserved.
# Tests for mt transforms

"test.concat.data.numeric" <- function()
{
    set.seed(10)
    testObs <- rnorm(nrow(iris)) > 0
    testIris <- iris[testObs,]
    trainIris <- iris[!testObs,]

    multiLogitOut <- rxLogisticRegression(
            formula = Species~Features, type = "multiClass", data = trainIris,
            mlTransforms = list(concat(vars = list(
                Features = c("Sepal.Length", "Sepal.Width", "Petal.Length", "Petal.Width")
              ))))
    sumOut <- summary(multiLogitOut)
    #checkEquals(names(coef(sumOut)),
        #c("0+intercept", "1+intercept", "2+intercept", "0+Petal.Width", 
          #"0+Petal.Length", "0+Sepal.Length","0+Sepal.Width", "1+Petal.Length",
          #"1+Sepal.Width", "2+Petal.Width", "2+Petal.Length"))
    
    # Test for invalid argument name  
    multiLogitOut2 <- try(
            rxLogisticRegression(
                formula = Species~Features, type = "multiClass", data = trainIris,
                mlTransforms = list(concat(vrs = list(
                    Features = c("Sepal.Length", "Sepal.Width", "Petal.Length", "Petal.Width")
              )))), silent = TRUE)
    checkTrue(inherits( multiLogitOut2, "try-error")) 
    checkTrue(grepl("is missing", as.character(multiLogitOut2)))         
           
} 

"test.mlTransforms.args.vars" <- function()
{
    claimsXdf <- file.path(rxGetOption("sampleDataDir"), "claims.xdf")
        
    claimsDF <- rxDataStep(inData = claimsXdf)
    claimsDF$cheap <- claimsDF$cost < 200
    claimsDF <- claimsDF[!is.na(claimsDF$cheap),]    
    claimsDF$ageStr <- as.character(claimsDF$age)
    claimsDF$typeStr <- as.character(claimsDF$type)
    claimsDF$carAgeStr <- as.character(claimsDF$car.age)
    lregOut1 <- rxLogisticRegression(cheap~catAge + catType, data = claimsDF,
         mlTransforms = list(featurizeText(vars = c(catAge = "ageStr", catType = "typeStr")))
        )
    coef1 <- coef(summary(lregOut1))
    
    lregOut2 <- rxLogisticRegression(cheap~catAge + catType, data = claimsDF,
      mlTransforms = list(featurizeText(vars = list(catAge = "ageStr", catType = "typeStr")))
    )
    coef2 <- coef(summary(lregOut2))
    checkEquals(coef1, coef2)

    lregOut3 <- rxLogisticRegression(cheap~ageTypeBag, data = claimsDF,
          mlTransforms = list(featurizeText(vars = list(ageTypeBag = c("ageStr", "typeStr"))))
            )

    coef3 <- coef(summary(lregOut3))  
    coef2Names <- names(coef2)
    coef2Names <- gsub("catAge.", "", coef2Names)
    coef2Names <- gsub("catType.", "", coef2Names)  
    names(coef2) <- coef2Names
    checkEquals(coef2[2], coef3[2], tolerance = 2E-2)
    checkEquals(coef2["(Bias)"], coef3["(Bias)"], tolerance = 1E-1)
}

"test.mlTransforms.multiple" <- function()
{
    claimsXdf <- file.path(rxGetOption("sampleDataDir"), "claims.xdf")
        
    claimsDF <- rxDataStep(inData = claimsXdf)
    claimsDF$cheap <- claimsDF$cost < 200
    claimsDF <- claimsDF[!is.na(claimsDF$cheap),]    
    claimsDF$ageStr <- as.character(claimsDF$age)
    claimsDF$typeStr <- as.character(claimsDF$type)
    claimsDF$carAgeStr <- as.character(claimsDF$car.age)
    lregOut1 <- rxLogisticRegression(cheap~catAge + catType + carAgeBag, data = claimsDF,
          mlTransforms = list(categorical(vars = c(catAge = "ageStr", catType = "typeStr")),
                              featurizeText(list(carAgeBag =  "carAgeStr"))))  
    lregSum <- summary(lregOut1)
    coefNames <- names(coef(lregSum))
    checkTrue("catType.A"  %in% coefNames)
    checkTrue("catAge.30-34"  %in% coefNames)
    checkTrue("carAgeBag.10"  %in% coefNames)
                        
    lregOut2 <- rxLogisticRegression(cheap~ageTypeBag + carAgeCat, data = claimsDF,
          mlTransforms = list(featurizeText(vars = list(ageTypeBag = c("ageStr", "typeStr"))),
                              categorical(vars = c(carAgeCat = "carAgeStr")))) 
    lRegSum2 <- summary(lregOut2)  
    coefNames2 <- names(coef(lRegSum2))
    checkTrue("ageTypeBag.a"  %in% coefNames2)
    checkTrue("ageTypeBag.60"  %in% coefNames2)
    checkTrue("carAgeCat.10+"  %in% coefNames2)
}

"test.categorical.data.character.ExpressEdition" <- function()
{
    utils.run.ExpressEdition(run.categorical.data.character.ExpressEdition)
}

"run.categorical.data.character.ExpressEdition" <- function()
{
    # Verifies that we are really running under Express Edition.
    checkTrue(rxIsExpressEdition())

    claimsXdf <- file.path(rxGetOption("sampleDataDir"), "claims.xdf")
        
    claimsDF <- rxDataStep(inData = claimsXdf)
    claimsDF$cheap <- claimsDF$cost < 200
    claimsDF <- claimsDF[!is.na(claimsDF$cheap),]    
    claimsDF$ageStr <- as.character(claimsDF$age)
    claimsDF$typeStr <- as.character(claimsDF$type)
    lregOut1 <- rxLogisticRegression(cheap~catAge + catType, data = claimsDF,
          mlTransforms = list(categorical(vars = c(catAge = "ageStr", catType = "typeStr"))))
   
    scoreOut1 <- rxPredict(lregOut1, data = claimsDF)
    
    lregOut2 <- rxLogisticRegression(cheap~age+type, data = claimsDF)
   
    scoreOut2 <- rxPredict(lregOut2, data = claimsDF)
    checkEquals(scoreOut1, scoreOut2)   
    
    ## specify mamlCode directly
    lregOut3 <- rxLogisticRegression(cheap~catAge + catType, data = claimsDF
        #, mlTransforms = list(categorical(vars = c(catAge = "ageStr", catType = "typeStr")))
        , mlTransformVars = c("ageStr", "typeStr")
        , mamlCode = "xf=cat{col=catAge:ageStr col=catType:typeStr}"
        )
    scoreOut3 <- rxPredict(lregOut3, data = claimsDF)
    summOut3 <- summary(lregOut3)
    checkEquals(scoreOut2, scoreOut3)

    ## combine concat and categorical
    lregOut4 <- rxLogisticRegression(cheap~cat, data = claimsDF
        , mlTransforms = list(
            concat(vars = list(str = c("ageStr", "typeStr"))), 
            categorical(vars = c(cat = "str")))
        #, mlTransformVars = c("ageStr", "typeStr")
        #, mamlCode = "xf=concat{col=str:ageStr,typeStr} xf=cat{col=cat:str}"
        )
    scoreOut4 <- rxPredict(lregOut4, data = claimsDF)
    summOut4 <- summary(lregOut4)
    checkEquals(scoreOut2, scoreOut4, tolerance = 1.3E-03)
    checkEquals(coef(summOut3), coef(summOut4), tolerance = 1.8E-02, checkNames = FALSE)
}

"test.categorical.data.character" <- function()
{
    if (rxIsExpressEdition())
        DEACTIVATED("Not running multi-thread testing for express edition.")

    claimsXdf <- file.path(rxGetOption("sampleDataDir"), "claims.xdf")
        
    claimsDF <- rxDataStep(inData = claimsXdf)
    claimsDF$cheap <- claimsDF$cost < 200
    claimsDF <- claimsDF[!is.na(claimsDF$cheap),]    
    claimsDF$ageStr <- as.character(claimsDF$age)
    claimsDF$typeStr <- as.character(claimsDF$type)
    lregOut1 <- rxLogisticRegression(cheap~catAge + catType, trainThreads = 8, data = claimsDF,
          mlTransforms = list(categorical(vars = c(catAge = "ageStr", catType = "typeStr"))))
   
    scoreOut1 <- rxPredict(lregOut1, data = claimsDF)
    
    lregOut2 <- rxLogisticRegression(cheap~age+type, trainThreads = 8, data = claimsDF)
   
    scoreOut2 <- rxPredict(lregOut2, data = claimsDF)
    checkEquals(scoreOut1, scoreOut2)   
    
    ## specify mamlCode directly
    lregOut3 <- rxLogisticRegression(cheap~catAge + catType, trainThreads = 8, data = claimsDF
        #, mlTransforms = list(categorical(vars = c(catAge = "ageStr", catType = "typeStr")))
        , mlTransformVars = c("ageStr", "typeStr")
        , mamlCode = "xf=cat{col=catAge:ageStr col=catType:typeStr}"
        )
    scoreOut3 <- rxPredict(lregOut3, data = claimsDF)
    summOut3 <- summary(lregOut3)
    checkEquals(scoreOut2, scoreOut3)

    ## combine concat and categorical
    lregOut4 <- rxLogisticRegression(cheap~cat, trainThreads = 8, data = claimsDF
        , mlTransforms = list(
            concat(vars = list(str = c("ageStr", "typeStr"))), 
            categorical(vars = c(cat = "str")))
        #, mlTransformVars = c("ageStr", "typeStr")
        #, mamlCode = "xf=concat{col=str:ageStr,typeStr} xf=cat{col=cat:str}"
        )
    scoreOut4 <- rxPredict(lregOut4, data = claimsDF)
    summOut4 <- summary(lregOut4)
    checkEquals(scoreOut2, scoreOut4, tolerance = 1E-05)
    checkEquals(coef(summOut3), coef(summOut4), tolerance = 1E-04, checkNames = FALSE)
}

"test.categoricalHash.data.character" <- function()
{
    claimsXdf <- file.path(rxGetOption("sampleDataDir"), "claims.xdf")
        
    claimsDF <- rxDataStep(inData = claimsXdf)
    claimsDF$cheap <- claimsDF$cost < 200
    # FIXME: Handling missing values
    claimsDF <- claimsDF[!is.na(claimsDF$cheap),] 	
    claimsDF$ageStr <- as.character(claimsDF$age)
    claimsDF$typeStr <- as.character(claimsDF$type)
    lregOut1 <- rxLogisticRegression(cheap~catAge + catType, trainThreads = 8, data = claimsDF,
          mlTransforms = list(categoricalHash(vars = c(catAge = "ageStr", catType = "typeStr"))))
   
    scoreOut1 <- rxPredict(lregOut1, data = claimsDF)
    lregOut2 <- rxLogisticRegression(cheap~age+type, trainThreads = 8, data = claimsDF)
    
    scoreOut2 <- rxPredict(lregOut2, data = claimsDF)
    # Results should be similar
    checkEquals(scoreOut1, scoreOut2, tolerance = 1E-02)   
 
    ## specify mamlCode directly
    lregOut3 <- rxLogisticRegression(cheap~catAge + catType, trainThreads = 8, data = claimsDF
        #, mlTransforms = list(categoricalHash(vars = c(catAge = "ageStr", catType = "typeStr")))
        , mlTransformVars = c("ageStr", "typeStr")
        , mamlCode = "xf=cathash{col=catAge:ageStr col=catType:typeStr}"
        )
    scoreOut3 <- rxPredict(lregOut3, data = claimsDF)
    summOut3 <- summary(lregOut3)
    checkEquals(scoreOut2, scoreOut3, tolerance = 1E-02)

    ## combine concat and categoricalHash
    lregOut4 <- rxLogisticRegression(cheap~cat, trainThreads = 8, data = claimsDF
        , mlTransforms = list(
            concat(vars = list(str = c("ageStr", "typeStr"))), 
            categoricalHash(vars = c(cat = "str")))
        #, mlTransformVars = c("ageStr", "typeStr")
        #, mamlCode = "xf=concat{col=str:ageStr,typeStr} xf=cathash{col=cat:str}"
        )
    scoreOut4 <- rxPredict(lregOut4, data = claimsDF)
    summOut4 <- summary(lregOut4)
    checkEquals(scoreOut2, scoreOut4, tolerance = 1E-02)

    #checkEquals(coef(summOut3), coef(summOut4), tolerance = 1E-05, checkNames = FALSE)
}

"test.categoricalHash.longTagName" <- function() 
{
    #DEACTIVATED("Bug 21712: Customer Issue: Change in mtCatHash.")

    mytext <- sapply(mapply(rep_len, x = letters[1:5], length = 470:479), paste, collapse = "")
    mytext <- rep(mytext, times = 5)
    myrating <- rep_len(0:1, length(mytext))
    mydata <- data.frame(Label = myrating, Text = mytext, stringsAsFactors = FALSE)

    mymodel <- rxLogisticRegression(formula = Label ~ Features, data = mydata,
        mlTransforms = list(categoricalHash(
            vars = c(Features = "Text")
            , hashBits = 8
            , invertHash = -1
            )))
        
    mycoef <- names(coef(mymodel))
    checkTrue(all(!duplicated(mycoef)))
}

"test.categorical.args.terms" <- function()
{
    testData <- data.frame(words =
        c("One", "Two", "One", "Two", "Three", "One", "Two", "Four", "One", "Two"),
        isOne = c(TRUE, FALSE, TRUE, FALSE, FALSE, TRUE, FALSE, FALSE, TRUE, FALSE),
        stringsAsFactors = FALSE)
    
    outModel1 <- rxLogisticRegression(isOne~words, data = testData, 
        mlTransforms = list(categorical(vars = "words", terms = c("One", "Two", "Three", "Four"))))
    summary(outModel1)
    
    # Use the model to score
    scoreOutDF1 <- rxPredict(outModel1, data = testData, extraVarsToWrite = c("isOne"))
    checkEquals(scoreOutDF1$isOne, scoreOutDF1$PredictedLabel)  
    
}


"test.concat.args" <- function()
{
    testObs <- rnorm(nrow(iris)) > 0
    testIris <- iris[testObs,]
    trainIris <- iris[!testObs,]
    multiLogit1 <- rxLogisticRegression(
        formula = Species~Sepal.Length + Sepal.Width + Petal.Length + Petal.Width,
        type = "multiClass", data = trainIris)
    sum1 <- summary(multiLogit1)
   
    multiLogit2 <- rxLogisticRegression(
        formula = Species~Features, type = "multiClass", data = trainIris,
        mlTransforms = list(concat(vars = list(Features = c("Sepal.Length", "Sepal.Width", "Petal.Length", "Petal.Width")))
          ))
    sum2 <- summary(multiLogit2)
    checkEquals(coef(sum1), coef(sum2))
} 

"test.mlTransforms.data.character" <- function()
{
 trainReviews <- data.frame(review = c( 
        "This is great",
        "I hate it",
        "Love it",
        "Do not like it",
        "Really like it",
        "I hate it",
        "I like it a lot",
        "I kind of hate it",
        "I do like it",
        "I really hate it",
        "It is very good",
        "I hate it a bunch",
        "I love it a bunch",
        "I hate it",
        "I like it very much",
        "I hate it very much.",
        "I really do love it",
        "I really do hate it",
        "Love it!",
        "Hate it!",
        "I love it",
        "I hate it",
        "I love it",
        "I hate it",
        "I love it"),
     like = c(TRUE, FALSE, TRUE, FALSE, TRUE,
        FALSE, TRUE, FALSE, TRUE, FALSE, TRUE, FALSE, TRUE,
        FALSE, TRUE, FALSE, TRUE, FALSE, TRUE, FALSE, TRUE, 
        FALSE, TRUE, FALSE, TRUE), stringsAsFactors = FALSE
    )

    testReviews <- data.frame(review = c(
        "This is great",
        "I hate it",
        "Love it",
        "Really like it",
        "I hate it",
        "I like it a lot",
        "I love it",
        "I do like it",
        "I really hate it",
        "I love it"), rowNum = 1:10, stringsAsFactors = FALSE)
 
    ############################################################################
    # categorical example
    ############################################################################
    # Use a categorical transform: the entire string is treated as a category
    outModel1 <- rxLogisticRegression(like~review, data = trainReviews, 
        mlTransforms = list(categorical(vars = "review")))
    # Note that 'I hate it' and 'I love it' (the only strings appearing more than once)
    # have non-zero weights
    summary(outModel1)
    
    # Use the model to score
    scoreOutDF1 <- rxPredict(outModel1, data = testReviews)
    scoreOutDF1$review<- testReviews$review
    scoreOutDF1
    
    ############################################################################
    # categoricalHash example
    ############################################################################  
    
    # Use a categorical hash transform
    outModel2 <- rxLogisticRegression(like~review, data = trainReviews, 
        mlTransforms = list(categoricalHash(vars = "review")))
    # Weights are similar to categorical
    summary(outModel2)
    
    # Use the model to score
    scoreOutDF2 <- rxPredict(outModel2, data = testReviews)
    scoreOutDF2$review<- testReviews$review
    scoreOutDF2   
    
    checkEquals(scoreOutDF1, scoreOutDF2, tolerance = 1E-03)

    ############################################################################
    # featurizeText example
    ############################################################################        
    outModel5 <- rxLogisticRegression(like~reviewHash, data = trainReviews, 
        mlTransforms = list(featurizeText(vars = c(reviewHash = "review"))))
    # 'hate' and 'love' have non-zero weights
    summary(outModel5)
    
    # Use the model to score
    scoreOutDF5 <- rxPredict(outModel5, data = testReviews)
    scoreOutDF5$review<- testReviews$review
    # Similar prediction results to mtWordBag
    scoreOutDF5    	
     
    
    ################################################################################
    # Simple example using user-specified transform
    ################################################################################
    myFormula <- isCase ~ age + parity + education + spontaneous + induced
    # Estimate a logistic regression model 
    logitModel1 <- rxLogisticRegression(formula = myFormula,
                           data = infert,
                           mlTransforms = list("Expr{col=isCase:case expr={x : x == 1}} "),
                           mlTransformVars = "case")

    ## specify mamlCode directly
    logitModel2 <- rxLogisticRegression(formula = myFormula,
                           data = infert,
                           mamlCode = "xf=Expr{col=isCase:case expr={x : x == 1}} ",
                           mlTransformVars = "case")
    checkEquals(coef(summary(logitModel1)), coef(summary(logitModel2)))

    ## use transforms
    logitModel3 <- rxLogisticRegression(formula = myFormula,
                           data = infert,
                           transforms = list(isCase = (case == 1)))
    checkEquals(coef(summary(logitModel1)), coef(summary(logitModel3)))
}

"test.getNetDefinition" <- function()
{
    model1 <- rxNeuralNet(
        formula = Species~Sepal.Length + Sepal.Width + Petal.Length + Petal.Width, 
        data = iris, 
        numHiddenNodes=10, 
        type="multi", 
        numIterations=10, 
        optimizer=adaDeltaSgd())
    model2 <- rxNeuralNet(
        formula = Species~Sepal.Length + Sepal.Width + Petal.Length + Petal.Width, 
        data = iris, 
        netDefinition=getNetDefinition(model1), 
        type="multi", 
        numIterations=0, 
        optimizer=adaDeltaSgd())
    
    checkTrue(length(coef(model1)) > 0)
    checkEquals(coef(model1), coef(model2))
    checkTrue(length(getNetDefinition(model1)) > 0)
    checkEquals(getNetDefinition(model1), getNetDefinition(model1))
    
    modelLr <- rxLogisticRegression(formula = Species~Sepal.Length + Sepal.Width + Petal.Length + Petal.Width, data = iris, type="multi")
    checkTrue(is.null(getNetDefinition(modelLr)))
}

"test.different.factor.levels.train.rxPredict" <- function()
{
	train <- data.frame(color = c(
		"red",
		"green",
		"blue",
		"cyan",
		"magenta",
		"yellow"),
		label = c(TRUE, FALSE, FALSE, FALSE, TRUE, TRUE))
	test <- data.frame(color = c(
		"red",
		"green",
		"blue",
		"cyan",
		"magenta",
		"orange"),
		label = c(TRUE, FALSE, FALSE, FALSE, TRUE, TRUE))
    model <- rxFastLinear(label ~ color, data = train, l2Weight = 0.01, l1Weight = 0.01, maxIterations = 10, checkFrequency = -1, trainThreads = 1, shuffle = FALSE)
	scores <- rxPredict(model, test, extraVarsToWrite="label")
	checkEquals(head(scores$label, 5), head(scores$PredictedLabel, 5))
	checkTrue(abs(0.5 - scores$Probability[[6]]) < 0.1)
}

"test.image.transforms" <- function()
{
    train <- data.frame(Path = c(
            system.file("help/figures/RevolutionAnalyticslogo.png", package = "MicrosoftML"),
            # REVIEW coeseanu: Seems bmps are not included with the package?!
            # system.file("help/figures/RevolutionAnalyticslogo.bmp", package = "MicrosoftML"),
            system.file("help/figures/RevolutionAnalyticslogo.jpg", package = "MicrosoftML")
        ), Label = c(1, 1), stringsAsFactors = FALSE)
    netDefFile <- netDefFile <- system.file("demoScripts/magic.nn", package = "MicrosoftML")
    model <- rxNeuralNet(
        Label ~ Features,
        type = "multi",
        data = train,
        mlTransforms = list(
            loadImage(vars = list(Features = "Path")),
            resizeImage(vars = "Features", width = 1, height = 1, resizing = "Aniso"),
            extractPixels(vars = "Features")
            ),
        mlTransformVars = "Path",
        normalize = "No",
        numIterations = 0,
        netDefinition = readChar(netDefFile, file.info(netDefFile)$size),
        postTransformCache = "Memory"
        )

    scores <- rxPredict(model, train)

    checkEquals(scores$Score.0, c(255, 254))
    checkEquals(scores$Score.1, c(255, 254))
    checkEquals(scores$Score.2, c(255, 254))
}

"test.sentiment.ep" <- function() {
    assign("useEntryPoints", TRUE, envir = MicrosoftML:::rxHashEnv)
    on.exit(assign("useEntryPoints", FALSE, envir = MicrosoftML:::rxHashEnv), add = TRUE)

    train <- data.frame(opinion = c(
        "I love it 1",
        "I love Ari",
        "Love it 1",
        "I love it a lot 1",
        "Really love it 1",
        "I hate it 2",
        "I hate it 2",
        "I hate it 2",
        "Hate it",
        "Hate"),
        like = rep(c(TRUE, FALSE), each = 5),
        stringsAsFactors = FALSE)

    # Train a dummy model so that we can use rxPredict and ask for
    # the 'opinion' column as an extra var.
    # REVIEW anro: I don't think we should have to specify 'opinion' here, but
    # sentiment is correctly populating mlTransformVars. Possibly bug in EP infra?
    model <- rxLogisticRegression(like ~ like, data = train,
      mlTransforms = list(getSentiment(vars = c(opinion = "opinion"))), mlTransformVars = "opinion")

    # Score the dummy model to force evaluation of the sentiment transform,
    # asking for the computed sentiment score as an out column.
    scores <- rxPredict(model, train, extraVarsToWrite = "opinion")$Probability

    # Check that for these obviously positive/negative sentences our model
    # scored correctly.
    checkEquals(train$like, scores > 0.5)
}

"test.sentiment" <- function() {
    train <- data.frame(opinion = c(
        "I love it 1",
        "I love Ari",
        "Love it 1",
        "I love it a lot 1",
        "Really love it 1",
        "I hate it 2",
        "I hate it 2",
        "I hate it 2",
        "Hate it",
        "Hate"),
        like = rep(c(TRUE, FALSE), each = 5),
        stringsAsFactors = FALSE)

    # Train a dummy model so that we can use rxPredict and ask for
    # the 'opinion' column as an extra var.
    model <- rxLogisticRegression(like ~ like, data = train,
      mlTransforms = list(getSentiment(vars = c(opinion = "opinion"))))

    # Score the dummy model to force evaluation of the sentiment transform,
    # asking for the computed sentiment score as an out column.
    scores <- rxPredict(model, train, extraVarsToWrite = "opinion")$opinion

    # Check that for these obviously positive/negative sentences our model
    # scored correctly.
    checkEquals(train$like, scores > 0.5)
}

"utils.run.ExpressEdition" <- function(testFunc)
{
    if (.Platform$OS!="windows")
        DEACTIVATED("Only windows platfrom has express edition.")
    # Whether the test runner was already in Express Edition
    wasExpressEdition = rxIsExpressEdition()
    # Set to run this test under Express Edition
    if (!wasExpressEdition)
        rxOptions(isExpressEdition = TRUE)
    # Verifies that we are really running under Express Edition.
    checkTrue(rxIsExpressEdition())
    tryCatch(do.call(testFunc, list()), finally = 
             {
                 # Restore original test runner setting regardless of whether
                 # test has passed or failed.
                 # Would be better to get rid of this if RUnit support
                 # test initializers and finalizers.
                 rxOptions(isExpressEdition = wasExpressEdition)
                 # Verify that the restoration of Express Edition setting 
                 # is successful. 
                 checkEquals(rxIsExpressEdition(), wasExpressEdition)
             })
}

