# Copyright (c) Microsoft Corporation.  All rights reserved.

# Tests for basic functionality of rxEnsemble

"test.rxEnsemble.basic.localseq" <- function() {
    cc <- rxSetComputeContext(RxLocalSeq())
    on.exit(rxSetComputeContext(cc))
    rxEnsemble.basic(trainers = list(fastTrees()))
    rxEnsemble.basic(trainers = list(fastForest(numLeaves = 15)))
    rxEnsemble.basic(trainers = list(fastLinear()))
    rxEnsemble.basic(trainers = list(logisticRegression()))
    rxEnsemble.basic(trainers = list(neuralNet()))
}

"test.rxEnsemble.basic.regression.localseq" <- function() {
    cc <- rxSetComputeContext(RxLocalSeq())
    on.exit(rxSetComputeContext(cc))
    rxEnsemble.basic.regression(trainers = list(fastTrees()))
    rxEnsemble.basic.regression(trainers = list(fastForest()))
    rxEnsemble.basic.regression(trainers = list(fastLinear()))
    rxEnsemble.basic.regression(trainers = list(neuralNet()))
}

"test.rxEnsemble.basic.multiclass.localseq" <- function() {
    cc <- rxSetComputeContext(RxLocalSeq())
    on.exit(rxSetComputeContext(cc))
    rxEnsemble.basic.multiclass(trainers = list(logisticRegression()))
    rxEnsemble.basic.multiclass(trainers = list(neuralNet()))
}

"test.rxEnsemble.basic.localpar" <- function() {
    cc <- rxSetComputeContext(RxLocalParallel())
    on.exit(rxSetComputeContext(cc))
    rxEnsemble.basic(trainers = list(fastTrees()))
    rxEnsemble.basic(trainers = list(fastForest()))
    rxEnsemble.basic(trainers = list(fastLinear()))
    rxEnsemble.basic(trainers = list(logisticRegression()))
    rxEnsemble.basic(trainers = list(neuralNet()))
}

"test.rxEnsemble.basic.regression.localpar" <- function() {
    cc <- rxSetComputeContext(RxLocalParallel())
    on.exit(rxSetComputeContext(cc))
    rxEnsemble.basic.regression(trainers = list(fastTrees()))
    rxEnsemble.basic.regression(trainers = list(fastForest()))
    rxEnsemble.basic.regression(trainers = list(fastLinear()))
    rxEnsemble.basic.regression(trainers = list(neuralNet()))
}

"test.rxEnsemble.basic.multiclass.localpar" <- function() {
    cc <- rxSetComputeContext(RxLocalParallel())
    on.exit(rxSetComputeContext(cc))
    rxEnsemble.basic.multiclass(trainers = list(logisticRegression()))
    rxEnsemble.basic.multiclass(trainers = list(neuralNet()))
}

"rxEnsemble.basic" <- function(trainers) {
    # Create temporary data sources
    myIris = iris
    myIris$Setosa <- iris$Species == "setosa"
    myXdf <- tempfile(pattern = "myIris", fileext = ".xdf")
    on.exit(file.remove(myXdf), add = TRUE)
    rxDataStep(myIris, myXdf, rowsPerRead = 50, overwrite = TRUE)
    myXdfDS <- RxXdfData(file = myXdf)

    myCsv <- tempfile(pattern = "myIris", fileext = ".csv")
    on.exit(file.remove(myCsv), add = TRUE)
    myCsvDS <- RxTextData(file = myCsv, colInfo = rxCreateColInfo(iris))
    rxDataStep(myIris, myCsvDS, overwrite = TRUE)

    # Run examples
    myFormula <- Setosa ~ Sepal.Length + Petal.Length + Sepal.Width

    # csv file
    ens <- rxEnsemble(formula = myFormula, data = myCsvDS, trainers = trainers, modelCount = 2, randomSeed = 42)
    checkTrue(!inherits(ens, "try-error"))
    checkTrue(class(ens)[[1]] == "Ensemble")
    scores <- rxPredict(ens, data = myXdfDS, extraVarsToWrite = c("Setosa"))
    roc <- rxRoc(actualVarName = "Setosa", predVarNames = "Probability", data = scores)
    auc <- rxAuc(roc)
    checkTrue(auc == 1)

    # xdf file
    ens <- rxEnsemble(formula = myFormula, data = myXdfDS, trainers = trainers, modelCount = 2, randomSeed = 42)
    checkTrue(!inherits(ens, "try-error"))
    checkTrue(class(ens)[[1]] == "Ensemble")
    scores <- rxPredict(ens, data = myXdfDS, extraVarsToWrite = c("Setosa"))
    roc <- rxRoc(actualVarName = "Setosa", predVarNames = "Probability", data = scores)
    auc <- rxAuc(roc)
    checkTrue(auc == 1)

    # data frame
    ens <- rxEnsemble(formula = myFormula, data = myIris, trainers = trainers, modelCount = 2, randomSeed = 42)
    checkTrue(!inherits(ens, "try-error"))
    checkTrue(class(ens)[[1]] == "Ensemble")
    scores <- rxPredict(ens, data = myXdfDS, extraVarsToWrite = c("Setosa"))
    roc <- rxRoc(actualVarName = "Setosa", predVarNames = "Probability", data = scores)
    auc <- rxAuc(roc)
    checkTrue(auc == 1)
}

"rxEnsemble.basic.regression" <- function(trainers) {
    # Create temporary data sources
    myMtcars <- mtcars
    myXdf <- tempfile(pattern = "myMtcars", fileext = ".xdf")
    on.exit(file.remove(myXdf), add = TRUE)
    rxDataStep(myMtcars, myXdf, rowsPerRead = 50, overwrite = TRUE)
    myXdfDS <- RxXdfData(file = myXdf)

    myCsv <- tempfile(pattern = "myMtcars", fileext = ".csv")
    on.exit(file.remove(myCsv), add = TRUE)
    myCsvDS <- RxTextData(file = myCsv, colInfo = rxCreateColInfo(mtcars))
    rxDataStep(myMtcars, myCsvDS, overwrite = TRUE)

    # Run examples
    myFormula <- mpg ~ cyl + disp + hp + drat + wt + qsec + vs + am + gear + carb

    # csv file
    ens <- rxEnsemble(formula = myFormula, data = myCsvDS, type = "regression", trainers = trainers, modelCount = 2)
    checkTrue(!inherits(ens, "try-error"))
    checkTrue(class(ens)[[1]] == "Ensemble")
    scores <- rxPredict(ens, data = myXdfDS, extraVarsToWrite = c("mpg"))

    # xdf file
    ens <- rxEnsemble(formula = myFormula, data = myXdfDS, type = "regression", trainers = trainers, modelCount = 2)
    checkTrue(!inherits(ens, "try-error"))
    checkTrue(class(ens)[[1]] == "Ensemble")
    scores <- rxPredict(ens, data = myXdfDS, extraVarsToWrite = c("mpg"))

    # data frame
    ens <- rxEnsemble(formula = myFormula, data = myMtcars, type = "regression", trainers = trainers, modelCount = 2)
    checkTrue(!inherits(ens, "try-error"))
    checkTrue(class(ens)[[1]] == "Ensemble")
    scores <- rxPredict(ens, data = myXdfDS, extraVarsToWrite = c("mpg"))
}

"rxEnsemble.basic.multiclass" <- function(trainers) {
    # Create temporary data sources
    myIris = iris
    myXdf <- tempfile(pattern = "myIris", fileext = ".xdf")
    on.exit(file.remove(myXdf), add = TRUE)
    rxDataStep(myIris, myXdf, rowsPerRead = 50, overwrite = TRUE)
    myXdfDS <- RxXdfData(file = myXdf)

    myCsv <- tempfile(pattern = "myIris", fileext = ".csv")
    on.exit(file.remove(myCsv), add = TRUE)
    myCsvDS <- RxTextData(file = myCsv, colInfo = rxCreateColInfo(iris))
    rxDataStep(myIris, myCsvDS, overwrite = TRUE)

    # Run examples
    myFormula <- Species ~ Sepal.Length + Petal.Length + Sepal.Width

    # csv file
    ens <- rxEnsemble(formula = myFormula, data = myCsvDS, trainers = trainers, type = "multiClass", modelCount = 2, randomSeed = 542)
    checkTrue(!inherits(ens, "try-error"))
    checkTrue(class(ens)[[1]] == "Ensemble")
    scores <- rxPredict(ens, data = myXdfDS, extraVarsToWrite = c("Species"))
    scores$Setosa <- ifelse(scores$Species == "setosa", 1, 0)
    roc <- rxRoc(actualVarName = "Setosa", predVarNames = "Score.setosa", data = scores)
    auc <- rxAuc(roc)
    checkTrue(auc >= 0.85)

    # xdf file
    ens <- rxEnsemble(formula = myFormula, data = myXdfDS, trainers = trainers, type = "multiClass", modelCount = 2, randomSeed = 542)
    checkTrue(!inherits(ens, "try-error"))
    checkTrue(class(ens)[[1]] == "Ensemble")
    scores <- rxPredict(ens, data = myXdfDS, extraVarsToWrite = c("Species"))
    scores$Versicolor <- ifelse(scores$Species == "versicolor", 1, 0)
    roc <- rxRoc(actualVarName = "Versicolor", predVarNames = "Score.versicolor", data = scores)
    auc <- rxAuc(roc)
    checkTrue(auc >= 0.5)

    # data frame
    ens <- rxEnsemble(formula = myFormula, data = myIris, trainers = trainers, type = "multiClass", modelCount = 2, randomSeed = 542)
    checkTrue(!inherits(ens, "try-error"))
    checkTrue(class(ens)[[1]] == "Ensemble")
    scores <- rxPredict(ens, data = myXdfDS, extraVarsToWrite = c("Species"))
    scores$Virginica <- ifelse(scores$Species == "virginica", 1, 0)
    roc <- rxRoc(actualVarName = "Virginica", predVarNames = "Score.virginica", data = scores)
    auc <- rxAuc(roc)
    checkTrue(auc >= 0.5)
}

"test.rxEnsemble.basic.anomaly" <- function(trainers) {
    myIris = iris
    myIris$isIris <- 1

    myXdf <- tempfile(pattern = "myIris", fileext = ".xdf")
    on.exit(file.remove(myXdf), add = TRUE)
    rxDataStep(myIris, myXdf, rowsPerRead = 50, overwrite = TRUE)
    myXdfDS <- RxXdfData(file = myXdf)

    myCsv <- tempfile(pattern = "myIris", fileext = ".csv")
    on.exit(file.remove(myCsv), add = TRUE)
    myCsvDS <- RxTextData(file = myCsv)
    rxDataStep(myIris, myCsvDS, overwrite = TRUE)

    readVars <- c("isIris", "Sepal.Length", "Sepal.Width", "Petal.Length", "Petal.Width")

    myFormula <- Species ~ Sepal.Length + Petal.Length + Sepal.Width

    irisExtra <- data.frame(
        Sepal.Length = c(2.5, 2.6, 82),
        Sepal.Width = c(.75, .9, 34),
        Petal.Length = c(2.5, 2.5, 99),
        Petal.Width = c(.8, .7, -38),
        isIris = 0,
        Species = "")

    irisScore <- iris[, c("Sepal.Length", "Sepal.Width", "Petal.Length", "Petal.Width")]
    irisScore$isIris <- 1
    irisScore$Species <- ""
    irisScore <- rbind(irisScore, irisExtra)

    # csv file
    ens <- rxEnsemble(formula = myFormula, data = myCsvDS, trainers = list(oneClassSvm()), type = "anomaly", modelCount = 2)
    checkTrue(!inherits(ens, "try-error"))
    checkTrue(class(ens)[[1]] == "Ensemble")
    scores <- rxPredict(ens, data = irisScore, extraVarsToWrite = c("isIris", "Score"), overwrite = TRUE)
    checkTrue(any(max.col(t(scores$Score)) == 151:153))

    # xdf file
    ens <- rxEnsemble(formula = myFormula, data = myXdfDS, trainers = list(oneClassSvm()), type = "anomaly", modelCount = 2)
    checkTrue(!inherits(ens, "try-error"))
    checkTrue(class(ens)[[1]] == "Ensemble")
    scores <- rxPredict(ens, data = irisScore, extraVarsToWrite = c("isIris", "Score"), overwrite = TRUE)
    checkTrue(any(max.col(t(scores$Score)) == 151:153))

    # data frame
    ens <- rxEnsemble(formula = myFormula, data = myIris, trainers = list(oneClassSvm()), type = "anomaly", modelCount = 2)
    checkTrue(!inherits(ens, "try-error"))
    checkTrue(class(ens)[[1]] == "Ensemble")
    scores <- rxPredict(ens, data = irisScore, extraVarsToWrite = c("isIris", "Score"), overwrite = TRUE)
    checkTrue(any(max.col(t(scores$Score)) == 151:153))
}

"test.rxEnsemble.split.data" <- function() {
    # Create temporary data sources
    myIris = iris
    myIris$Setosa <- iris$Species == "setosa"
    myXdf <- tempfile(pattern = "myIris", fileext = ".xdf")
    on.exit(file.remove(myXdf), add = TRUE)
    rxDataStep(myIris, myXdf, rowsPerRead = 50, overwrite = TRUE)
    myXdfDS <- RxXdfData(file = myXdf)

    # Run examples
    myFormula <- Setosa ~ Sepal.Length + Petal.Length + Sepal.Width

    # xdf
    ens <- rxEnsemble(formula = myFormula, data = list(myXdfDS, myXdfDS), trainers = list(fastForest(), fastForest(numLeaves = 15)), modelCount = 2, randomSeed = 42)
    checkTrue(!inherits(ens, "try-error"))
    checkTrue(class(ens)[[1]] == "Ensemble")
    summ <- summary(ens)
    CheckStringsFound(summ, c("Partition model 0 summary:", "Partition model 1 summary:", "Per-feature gain summary"))
    checkTrue(length(grep("Partition model 2 summary:", summ)) == 0)
    scores <- rxPredict(ens, data = myXdfDS, extraVarsToWrite = c("Setosa"))
    roc <- rxRoc(actualVarName = "Setosa", predVarNames = "Probability", data = scores)
    auc <- rxAuc(roc)
    checkTrue(auc == 1)

    ens <- rxEnsemble(formula = myFormula, data = list(myXdfDS, myXdfDS), trainers = list(fastLinear(trainThreads = 1), logisticRegression()), modelCount = 2, randomSeed = 42)
    checkTrue(!inherits(ens, "try-error"))
    checkTrue(class(ens)[[1]] == "Ensemble")
    summ <- summary(ens)
    CheckStringsFound(summ, c("Partition model 0 summary:", "Partition model 1 summary:", "Linear Binary Classification Predictor non-zero weights"))
    checkTrue(length(grep("Partition model 2 summary:", summ)) == 0)
    scores <- rxPredict(ens, data = myXdfDS, extraVarsToWrite = c("Setosa"))
    roc <- rxRoc(actualVarName = "Setosa", predVarNames = "Probability", data = scores)
    auc <- rxAuc(roc)
    checkTrue(auc == 1)

    ens <- rxEnsemble(formula = myFormula, data = list(myXdfDS, myXdfDS), trainers = list(neuralNet(), fastTrees(numLeaves = 15)), modelCount = 2, randomSeed = 42)
    checkTrue(!inherits(ens, "try-error"))
    checkTrue(class(ens)[[1]] == "Ensemble")
    summ <- summary(ens)
    CheckStringsFound(summ, c("Partition model 0 summary:", "Partition model 1 summary:", "Input Layers", "Per-feature gain summary"))
    checkTrue(length(grep("Partition model 2 summary:", summ)) == 0)
    scores <- rxPredict(ens, data = myXdfDS, extraVarsToWrite = c("Setosa"))
    roc <- rxRoc(actualVarName = "Setosa", predVarNames = "Probability", data = scores)
    auc <- rxAuc(roc)
    checkTrue(auc == 1)

    # mixture
    cc <- rxSetComputeContext('localpar')
    on.exit(rxSetComputeContext(cc))
    ens <- rxEnsemble(formula = myFormula, data = list(myXdfDS, myIris), trainers = list(fastTrees(), fastLinear(trainThreads = 1)), modelCount = 2, randomSeed = 42)
    checkTrue(!inherits(ens, "try-error"))
    checkTrue(class(ens)[[1]] == "Ensemble")
    summ <- summary(ens)
    CheckStringsFound(summ, c("Partition model 0 summary:", "Partition model 1 summary:", "Linear Binary Classification Predictor non-zero weights", "Per-feature gain summary"))
    checkTrue(length(grep("Partition model 2 summary:", summ)) == 0)
    scores <- rxPredict(ens, data = myXdfDS, extraVarsToWrite = c("Setosa"))
    roc <- rxRoc(actualVarName = "Setosa", predVarNames = "Probability", data = scores)
    auc <- rxAuc(roc)
    checkTrue(auc == 1)
}

"test.rxEnsemble.regression.split.data" <- function() {
    # Create temporary data sources
    myMtcars <- mtcars
    myXdf <- tempfile(pattern = "myMtcars", fileext = ".xdf")
    on.exit(file.remove(myXdf), add = TRUE)
    rxDataStep(myMtcars, myXdf, rowsPerRead = 50, overwrite = TRUE)
    myXdfDS <- RxXdfData(file = myXdf)

    myFormula <- mpg ~ cyl + disp + hp + drat + wt + qsec + vs + am + gear + carb

    # xdf
    ens <- rxEnsemble(formula = myFormula, data = list(myXdfDS, myXdfDS), type = "regression", trainers = list(fastForest(), fastForest(numLeaves = 15)), modelCount = 2)
    checkTrue(!inherits(ens, "try-error"))
    checkTrue(class(ens)[[1]] == "Ensemble")
    scores <- rxPredict(ens, data = myXdfDS, extraVarsToWrite = c("mpg"))

    ens <- rxEnsemble(formula = myFormula, data = list(myXdfDS, myXdfDS), type = "regression", trainers = list(fastLinear(trainThreads = 1), neuralNet()), modelCount = 2)
    checkTrue(!inherits(ens, "try-error"))
    checkTrue(class(ens)[[1]] == "Ensemble")
    scores <- rxPredict(ens, data = myXdfDS, extraVarsToWrite = c("mpg"))

    ens <- rxEnsemble(formula = myFormula, data = list(myXdfDS, myXdfDS), type = "regression", trainers = list(neuralNet(), fastTrees(numLeaves = 15)), modelCount = 2)
    checkTrue(!inherits(ens, "try-error"))
    checkTrue(class(ens)[[1]] == "Ensemble")
    scores <- rxPredict(ens, data = myXdfDS, extraVarsToWrite = c("mpg"))

    # mixture
    cc <- rxSetComputeContext('localpar')
    on.exit(rxSetComputeContext(cc))
    ens <- rxEnsemble(formula = myFormula, data = list(myXdfDS, myMtcars), type = "regression", trainers = list(fastTrees(), fastLinear(trainThreads = 1)), modelCount = 2)
    checkTrue(!inherits(ens, "try-error"))
    checkTrue(class(ens)[[1]] == "Ensemble")
    scores <- rxPredict(ens, data = myXdfDS, extraVarsToWrite = c("mpg"))
}

"test.rxEnsemble.binary.get.original.label" <- function() {
    myIris = iris
    myIris$TextLabel = ifelse(myIris$Species == "setosa", "setosa", "notsetosa")
    myIris$BoolLabel = ifelse(myIris$Species == "setosa", TRUE, FALSE)
    myIris$NumLabel = ifelse(myIris$Species == "setosa", 4, 6)

    myXdf <- tempfile(pattern = "myIris", fileext = ".xdf")
    on.exit(file.remove(myXdf), add = TRUE)
    rxDataStep(myIris, myXdf, rowsPerRead = 50, overwrite = TRUE)
    myXdfDS <- RxXdfData(file = myXdf)

    # Text label
    myFormula <- TextLabel ~ Sepal.Length + Petal.Length + Sepal.Width
    ens <- rxEnsemble(formula = myFormula, data = myXdfDS, trainers = list(fastTrees()), modelCount = 2)
    checkTrue(!inherits(ens, "try-error"))
    checkTrue(class(ens)[[1]] == "Ensemble")
    scores <- rxPredict(ens, data = myXdfDS)
    colNames <- names(scores)
    checkTrue(colNames[[1]] == "PredictedLabel" && colNames[[2]] == "Score.setosa" && colNames[[3]] == "Probability.setosa")

    # Bool label
    myFormula <- BoolLabel ~ Sepal.Length + Petal.Length + Sepal.Width
    ens <- rxEnsemble(formula = myFormula, data = myXdfDS, trainers = list(fastTrees()), modelCount = 2)
    checkTrue(!inherits(ens, "try-error"))
    checkTrue(class(ens)[[1]] == "Ensemble")
    scores <- rxPredict(ens, data = myXdfDS)
    colNames <- names(scores)
    checkTrue(colNames[[1]] == "PredictedLabel" && colNames[[2]] == "Score" && colNames[[3]] == "Probability")

    # Numeric label
    myFormula <- NumLabel ~ Sepal.Length + Petal.Length + Sepal.Width
    ens <- rxEnsemble(formula = myFormula, data = myXdfDS, trainers = list(fastTrees()), modelCount = 2)
    checkTrue(!inherits(ens, "try-error"))
    checkTrue(class(ens)[[1]] == "Ensemble")
    scores <- rxPredict(ens, data = myXdfDS)
    colNames <- names(scores)
    checkTrue(colNames[[1]] == "PredictedLabel" && colNames[[2]] == "Score.6" && colNames[[3]] == "Probability.6")
}

"test.rxEnsemble.multiclass.split.data" <- function() {
    # Create temporary data sources
    myIris = iris
    myXdf <- tempfile(pattern = "myIris", fileext = ".xdf")
    on.exit(file.remove(myXdf), add = TRUE)
    rxDataStep(myIris, myXdf, rowsPerRead = 50, overwrite = TRUE)
    myXdfDS <- RxXdfData(file = myXdf)

    # Run examples
    myFormula <- Species ~ Sepal.Length + Petal.Length + Sepal.Width

    # xdf
    ens <- rxEnsemble(formula = myFormula, data = list(myXdfDS, myXdfDS), trainers = list(neuralNet(), neuralNet(numIterations = 50)), type = "multiClass", modelCount = 2)
    checkTrue(!inherits(ens, "try-error"))
    checkTrue(class(ens)[[1]] == "Ensemble")
    scores <- rxPredict(ens, data = myXdfDS, extraVarsToWrite = c("Species"))
    scores$Virginica <- ifelse(scores$Species == "virginica", 1, 0)
    roc <- rxRoc(actualVarName = "Virginica", predVarNames = "Score.virginica", data = scores)
    auc <- rxAuc(roc)
    checkTrue(auc >= 0.5)

    ens <- rxEnsemble(formula = myFormula, data = list(myXdfDS, myXdfDS), trainers = list(logisticRegression(l2Weight = 0.5), logisticRegression()), type = "multiClass", modelCount = 2)
    checkTrue(!inherits(ens, "try-error"))
    checkTrue(class(ens)[[1]] == "Ensemble")
    scores <- rxPredict(ens, data = myXdfDS, extraVarsToWrite = c("Species"))
    scores$Versicolor <- ifelse(scores$Species == "versicolor", 1, 0)
    roc <- rxRoc(actualVarName = "Versicolor", predVarNames = "Score.versicolor", data = scores)
    auc <- rxAuc(roc)
    checkTrue(auc >= 0.96)

    ens <- rxEnsemble(formula = myFormula, data = list(myXdfDS, myXdfDS), trainers = list(neuralNet(), logisticRegression(memorySize = 30)), type = "multiClass", modelCount = 2)
    checkTrue(!inherits(ens, "try-error"))
    checkTrue(class(ens)[[1]] == "Ensemble")
    scores <- rxPredict(ens, data = myXdfDS, extraVarsToWrite = c("Species"))
    scores$Virginica <- ifelse(scores$Species == "virginica", 1, 0)
    roc <- rxRoc(actualVarName = "Virginica", predVarNames = "Score.virginica", data = scores)
    auc <- rxAuc(roc)
    checkTrue(auc >= 0.98)

    # mixture
    cc <- rxSetComputeContext('localpar')
    on.exit(rxSetComputeContext(cc))
    ens <- rxEnsemble(formula = myFormula, data = list(myXdfDS, myIris), trainers = list(logisticRegression(), neuralNet()), type = "multiClass", modelCount = 2)
    checkTrue(!inherits(ens, "try-error"))
    checkTrue(class(ens)[[1]] == "Ensemble")
    scores <- rxPredict(ens, data = myXdfDS, extraVarsToWrite = c("Species"))
    scores$Virginica <- ifelse(scores$Species == "virginica", 1, 0)
    roc <- rxRoc(actualVarName = "Virginica", predVarNames = "Score.virginica", data = scores)
    auc <- rxAuc(roc)
    checkTrue(auc >= 0.98)
}

"test.rxEnsemble.anomaly.split.data" <- function() {
    myIris = iris
    myIris$isIris <- 1

    myXdf <- tempfile(pattern = "myIris", fileext = ".xdf")
    on.exit(file.remove(myXdf), add = TRUE)
    rxDataStep(myIris, myXdf, rowsPerRead = 50, overwrite = TRUE)
    myXdfDS <- RxXdfData(file = myXdf)

    myCsv <- tempfile(pattern = "myIris", fileext = ".csv")
    on.exit(file.remove(myCsv), add = TRUE)
    myCsvDS <- RxTextData(file = myCsv)
    rxDataStep(myIris, myCsvDS, overwrite = TRUE)

    readVars <- c("isIris", "Sepal.Length", "Sepal.Width", "Petal.Length", "Petal.Width")

    myFormula <- Species ~ Sepal.Length + Petal.Length + Sepal.Width

    irisExtra <- data.frame(
        Sepal.Length = c(2.5, 2.6, 82),
        Sepal.Width = c(.75, .9, 34),
        Petal.Length = c(2.5, 2.5, 99),
        Petal.Width = c(.8, .7, -38),
        isIris = 0,
        Species = "")

    irisScore <- iris[, c("Sepal.Length", "Sepal.Width", "Petal.Length", "Petal.Width")]
    irisScore$isIris <- 1
    irisScore$Species <- ""
    irisScore <- rbind(irisScore, irisExtra)

    # xdf
    ens <- rxEnsemble(formula = myFormula, data = list(myXdfDS, myXdfDS), trainers = list(oneClassSvm(), oneClassSvm(kernel = sigmoidKernel())), type = "anomaly", modelCount = 2)
    checkTrue(!inherits(ens, "try-error"))
    checkTrue(class(ens)[[1]] == "Ensemble")
    scores <- rxPredict(ens, data = irisScore, extraVarsToWrite = c("isIris", "Score"), overwrite = TRUE)
    checkTrue(any(max.col(t(scores$Score)) == 151:153))

    ens <- rxEnsemble(formula = myFormula, data = list(myXdfDS, myXdfDS), trainers = list(oneClassSvm(kernel = polynomialKernel()), oneClassSvm(epsilon = 0.001)), type = "anomaly", modelCount = 2)
    checkTrue(!inherits(ens, "try-error"))
    checkTrue(class(ens)[[1]] == "Ensemble")
    scores <- rxPredict(ens, data = irisScore, extraVarsToWrite = c("isIris", "Score"), overwrite = TRUE)
    checkTrue(any(max.col(t(scores$Score)) == 151:153))

    ens <- rxEnsemble(formula = myFormula, data = list(myXdfDS, myXdfDS), trainers = list(oneClassSvm(nu = 0.3), oneClassSvm(kernel = linearKernel())), type = "anomaly", modelCount = 2)
    checkTrue(!inherits(ens, "try-error"))
    checkTrue(class(ens)[[1]] == "Ensemble")
    scores <- rxPredict(ens, data = irisScore, extraVarsToWrite = c("isIris", "Score"), overwrite = TRUE)
    checkTrue(any(max.col(t(scores$Score)) == 151:153))

    # mixture
    cc <- rxSetComputeContext('localpar')
    on.exit(rxSetComputeContext(cc))
    ens <- rxEnsemble(formula = myFormula, data = list(myXdfDS, myIris), trainers = list(oneClassSvm(nu = 0.5, epsilon = 0.1), oneClassSvm()), type = "anomaly", modelCount = 2)
    checkTrue(!inherits(ens, "try-error"))
    checkTrue(class(ens)[[1]] == "Ensemble")
    scores <- rxPredict(ens, data = irisScore, extraVarsToWrite = c("isIris", "Score"), overwrite = TRUE)
    checkTrue(any(max.col(t(scores$Score)) == 151:153))
}

"test.pipeline.ensemble" <- function() {
    data1 <- data.frame(Label = c(1, 1, 1, 0, 0, 0), Cat = c("a", "a", "a", "b", "b", "b"), stringsAsFactors = FALSE)
    data2 <- data.frame(Label = c(1, 1, 1, 0, 0, 0), Cat = c("c", "c", "c", "d", "d", "d"), stringsAsFactors = FALSE)
    dataTest <- data.frame(Label = c(5, 5, 5, 5), Cat = c("a", "b", "c", "d"), stringsAsFactors = FALSE)

    formula <- Label ~ Features
    mlTransforms <- list(categorical(vars = c(Features = "Cat")))

    # Test average.
    ensemble <- rxEnsemble(formula = formula, data = list(data1, data2), trainers = list(fastLinear(trainThreads = 1, shuffle = FALSE)), modelCount = 2, mlTransforms = mlTransforms, combineMethod = "average")
    linear1 <- rxFastLinear(data = data1, formula = formula, trainThreads = 1, mlTransforms = mlTransforms, shuffle = FALSE)
    linear2 <- rxFastLinear(data = data2, formula = formula, trainThreads = 1, mlTransforms = mlTransforms, shuffle = FALSE)

    ensembleScores <- rxPredict(ensemble, data = dataTest)
    linear1Scores <- rxPredict(linear1, data = dataTest)
    linear2Scores <- rxPredict(linear2, data = dataTest)
    ensembleScores$average <- (linear1Scores$Score.1 + linear2Scores$Score.1) / 2
    checkTrue(all(abs(ensembleScores$Score - (linear1Scores$Score.1 + linear2Scores$Score.1) / 2) < 0.000001))

    # Test median.
    data3 <- data.frame(Label = c(1, 1, 1, 0, 0, 0), Cat = c("e", "e", "e", "f", "f", "f"), stringsAsFactors = FALSE)
    ensemble <- rxEnsemble(formula = formula, data = list(data1, data2, data3), trainers = list(fastLinear(trainThreads = 1)), modelCount = 3, mlTransforms = mlTransforms)
    linear1 <- rxFastLinear(data = data1, formula = formula, trainThreads = 1, mlTransforms = mlTransforms, shuffle = FALSE)
    linear2 <- rxFastLinear(data = data2, formula = formula, trainThreads = 1, mlTransforms = mlTransforms, shuffle = FALSE)
    linear3 <- rxFastLinear(data = data3, formula = formula, trainThreads = 1, mlTransforms = mlTransforms, shuffle = FALSE)
    ensembleScores <- rxPredict(ensemble, data = dataTest)
    linear1Scores <- rxPredict(linear1, data = dataTest)
    linear2Scores <- rxPredict(linear2, data = dataTest)
    linear3Scores <- rxPredict(linear3, data = dataTest)
    checkTrue(all(ensembleScores$Score == linear3Scores$Score.1))

    # Test same for regression
    ensemble <- rxEnsemble(formula = formula, data = list(data1, data2), trainers = list(fastTrees(numTrees = 2, numLeaves = 4, minSplit = 1)), type = "regression", modelCount = 2, mlTransforms = mlTransforms, combineMethod = "average")
    tree1 <- rxFastTrees(data = data1, formula = formula, type = "regression", numTrees = 2, numLeaves = 4, minSplit = 1, mlTransforms = mlTransforms)
    tree2 <- rxFastTrees(data = data2, formula = formula, type = "regression", numTrees = 2, numLeaves = 4, minSplit = 1, mlTransforms = mlTransforms)

    ensembleScores <- rxPredict(ensemble, data = dataTest)
    tree1Scores <- rxPredict(tree1, data = dataTest)
    tree2Scores <- rxPredict(tree2, data = dataTest)
    ensembleScores$average <- (tree1Scores$Score + tree2Scores$Score) / 2
    checkTrue(all(abs(ensembleScores$Score - (tree1Scores$Score + tree2Scores$Score) / 2) < 0.0000001))

    ensemble <- rxEnsemble(formula = formula, data = list(data1, data2, data3), trainers = list(fastTrees(numTrees = 2, numLeaves = 4, minSplit = 1)), type = "regression", modelCount = 3, mlTransforms = mlTransforms)
    tree1 <- rxFastTrees(data = data1, formula = formula, numTrees = 2, numLeaves = 4, minSplit = 1, type = "regression", mlTransforms = mlTransforms)
    tree2 <- rxFastTrees(data = data2, formula = formula, numTrees = 2, numLeaves = 4, minSplit = 1, type = "regression", mlTransforms = mlTransforms)
    tree3 <- rxFastTrees(data = data3, formula = formula, numTrees = 2, numLeaves = 4, minSplit = 1, type = "regression", mlTransforms = mlTransforms)
    ensembleScores <- rxPredict(ensemble, data = dataTest)
    tree1Scores <- rxPredict(tree1, data = dataTest)
    tree2Scores <- rxPredict(tree2, data = dataTest)
    tree3Scores <- rxPredict(tree3, data = dataTest)
    checkTrue(all(ensembleScores$Score == tree3Scores$Score))

    # Test for multiclass
    data1 <- data.frame(Label = c(1, 1, 1, 0, 0, 0, 2, 2, 2), Cat = c("a", "a", "a", "b", "b", "b", "g", "g", "g"), stringsAsFactors = FALSE)
    data2 <- data.frame(Label = c(1, 1, 1, 0, 0, 0, 2, 2, 2), Cat = c("c", "c", "c", "d", "d", "d", "h", "h", "h"), stringsAsFactors = FALSE)
    data3 <- data.frame(Label = c(1, 1, 1, 0, 0, 0, 2, 2, 2), Cat = c("e", "e", "e", "f", "f", "f", "i", "i", "i"), stringsAsFactors = FALSE)
    dataTest <- data.frame(Label = c(5, 5, 5, 5, 5, 5), Cat = c("a", "b", "c", "d", "g", "h"), stringsAsFactors = FALSE)

    ensemble <- rxEnsemble(formula = formula, data = list(data1, data2), trainers = list(logisticRegression()), type = "multiClass", modelCount = 2, mlTransforms = mlTransforms, combineMethod = "average")
    lr1 <- rxLogisticRegression(data = data1, formula = formula, type = "multiClass", mlTransforms = mlTransforms)
    lr2 <- rxLogisticRegression(data = data2, formula = formula, type = "multiClass", mlTransforms = mlTransforms)

    ensembleScores <- rxPredict(ensemble, data = dataTest)
    lr1Scores <- rxPredict(lr1, data = dataTest)
    lr2Scores <- rxPredict(lr2, data = dataTest)
    ensembleScores$average.0 <- (lr1Scores$Score.0 + lr2Scores$Score.0) / 2
    ensembleScores$average.1 <- (lr1Scores$Score.1 + lr2Scores$Score.1) / 2
    ensembleScores$average.2 <- (lr1Scores$Score.2 + lr2Scores$Score.2) / 2
    checkTrue(all(abs(ensembleScores$Score.0 - (lr1Scores$Score.0 + lr2Scores$Score.0) / 2) < 0.0000001))
}


"test.rxEnsemble.binary.transform" <- function() {
    #library(MicrosoftML)

    dataFile <- file.path(rxGetOption("sampleDataDir"), "claims4blocks.xdf")
    dataSource <- RxXdfData(dataFile)

    transforms <- quote(list(age30 = (age == "30-34" | age == "35-39")))
    form <- age30 ~ car.age + type + cost + number

    #rxFastTrees(form, data = dataSource, type = "binary", transforms = transforms)  #OK

    rxEnsemble(form, data = dataSource, type = "binary", transforms = transforms,
        , trainers = list(fastTrees(randomSeed = 100), fastTrees(numTrees = 60), fastTrees(learningRate = 0.1))
        ) #Error: *** Exception: 'data must have a Label column

    checkTrue(TRUE)
}

"test.rxEnsemble.regression.variable" <- function() {
    #library(MicrosoftML)

    dataFile <- file.path(rxGetOption("sampleDataDir"), "claims4blocks.xdf")
    dataSource <- RxXdfData(dataFile)

    form <- cost ~ car.age + type + number
    randomSeed <- 100
    trainers = list(fastTrees(randomSeed = randomSeed), fastTrees(numTrees = 60), fastTrees(learningRate = 0.1))

    #rxEnsemble(form, data = dataSource, type = "regression", transforms = transforms, 
        #, trainers = list(fastTrees(randomSeed = 100), fastTrees(numTrees = 60), fastTrees(learningRate = 0.1))
        #)   #OK

    rxEnsemble(form, data = dataSource, type = "regression", #transforms = transforms, 
        , trainers = trainers
        ) #Error in grep("^\\$.*\\$$", arg) : object 'randomSeed' not found

    checkTrue(TRUE)
}

"test.rxEnsemble.regression.randomSeed" <- function() {
    #library(MicrosoftML)
    colInfo <- list(DayOfWeek = list(type = "factor", levels = c("Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday")))
    source <- system.file("SampleData/AirlineDemoSmall.csv", package = "RevoScaleR")
    data <- RxTextData(source, missingValueString = "M", colInfo = colInfo)

    ensemble <- rxEnsemble(formula = ArrDelay ~ DayOfWeek, data = data, type = "regression", replace = TRUE, randomSeed = 100,
        trainers = list(fastTrees(randomSeed = 200), fastForest(randomSeed = 300, numTrees = 60), fastLinear(convergenceTolerance = 0.1))
        )
    checkTrue(TRUE)
}

"test.rxEnsemble.r.transforms" <- function() {
    inData <- RxXdfData(file.path(rxGetOption("sampleDataDir"), "claims.xdf"))
    form <- ageLogical ~ car.age + type + cost + number
    trainers = list(fastTrees(), fastTrees(numTrees = 60), fastTrees(learningRate = 0.1))
    rxEnsemble(formula = form, data = inData, type = 'binary', transforms = list(ageLogical = (age == "30-34")), trainers = trainers)

    transforms <- quote(list(ageLogical = (age == "30-34")))
    rowSelection <- quote(number > 10)

    rxEnsemble(formula = form, data = inData, trainers = trainers, transforms = transforms)
    rxEnsemble(formula = form, data = inData, trainers = trainers, transforms = transforms, rowSelection = rowSelection)
}

"test.rxEnsemble.error.report" <- function() {
    DEACTIVATED("Failed on all three Linux platforms: Error in checkTrue(length(l) == 6) : Test not TRUE")

    myIris = iris
    myFormula <- Species ~ Sepal.Length + Petal.Length + Sepal.Width
    output <- capture.output(try(rxEnsemble(formula = myFormula, data = myIris, trainers = list(fastTrees()), modelCount = 3)))

    outputConcatenated <- paste(output, collapse = "")
    string <- paste0("Exception: 'The label column 'Species' of the training data has more than two classes. Only two classes are allowed for binary classification.\r",
        "Parameter name: data'",
        "Error in processing machine learning request.", sep = '')
    g <- gregexpr(string, outputConcatenated)
    l <- attr(g[[1]], "match.length")
    checkTrue(length(l) == 6)
}

# Helper function for unit tests.
"CheckStringsFound" <- function(outputList, stringsToFind) {
    outputConcatenated <- paste(outputList, collapse = "")
    for (string in stringsToFind) {
        checkTrue(length(grep(string, outputConcatenated)) > 0)
    }
}

"test.rxEnsemble.calibration.arg" <- function() {
    myIris = iris
    myIris$Setosa <- iris$Species == "setosa"
    myFormula <- Setosa ~ Sepal.Length + Petal.Length + Sepal.Width

    ens1 <- rxEnsemble(
        formula = myFormula, data = myIris, 
        trainers = list(fastTrees(trainThreads = 1), fastLinear(trainThreads = 1)), modelCount = 2, randomSeed = 42, maxCalibration = 120)
    ens2 <- rxEnsemble(
        formula = myFormula, data = myIris, 
        trainers = list(fastTrees(trainThreads = 1), fastLinear(trainThreads = 1)), modelCount = 2, randomSeed = 42, maxCalibration = 85)
    ens3 <- rxEnsemble(
        formula = myFormula, data = myIris,
        trainers = list(fastTrees(trainThreads = 1), fastLinear(trainThreads = 1)), modelCount = 2, randomSeed = 42, maxCalibration = 40)
    ens4 <- rxEnsemble(
        formula = myFormula, data = myIris,
        trainers = list(fastTrees(trainThreads = 1), fastLinear(trainThreads = 1)), modelCount = 2, randomSeed = 42, maxCalibration = 0)
    score1 <- rxPredict(ens1, data = myIris)
    score2 <- rxPredict(ens2, data = myIris)
    score3 <- rxPredict(ens3, data = myIris)
    score4 <- rxPredict(ens4, data = myIris)

    # The ensemble models should be identical, but with different calibrators, so the scores should be the same for all four, and
    # the probabilities should be different. score4 should not have a probability column.
    score1$ScoreDiff <- abs(score1$Score - score2$Score)
    score1$ProbDiff <- abs(score1$Probability - score2$Probability)
    checkTrue(all(score1$ScoreDiff == 0))
    checkTrue(all(score1$ProbDiff != 0))

    score1$ScoreDiff <- abs(score1$Score - score3$Score)
    score1$ProbDiff <- abs(score1$Probability - score3$Probability)
    checkTrue(all(score1$ScoreDiff == 0))
    checkTrue(all(score1$ProbDiff != 0))

    score1$ProbDiff <- abs(score2$Probability - score3$Probability)
    checkTrue(all(score1$ProbDiff != 0))

    score1$ScoreDiff <- abs(score1$Score - score4$Score)
    checkTrue(all(score1$ScoreDiff == 0))
    checkTrue(is.null(score4$Probability))
}
